/*
 * random_seed.h
 *
 * Copyright (c) 2013 Metaparadigm Pte. Ltd.
 * Michael Clark <michael@metaparadigm.com>
 *
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the MIT license. See COPYING for details.
 *
 */

#ifndef seed_h
#define seed_h

#ifdef __cplusplus
extern "C" {
#endif

extern int json_c_get_random_seed();

#ifdef __cplusplus
}
#endif

#endif
